% The LTE proportional fair scheduling scheme described in "LTE - The UMTS
% Long Term Evolution: From Theory to Practice", 2009, pp. 291

% window: the time window over which fairness is imposed (t_c in the
%         reference)

function [station,UE,AttachList] = PF_scheduler_LTE(station,UE,NumPRB,NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots,window)

AttachList = find([UE.attach]==station.NameTag);
AvaTimeSlots = sum(station.ABS(1:NumTimeSlots)); % Available subframes (time slots), less than or equal to NumTimeSlots due to ABS
NumUE = length(AttachList);

if AvaTimeSlots > 0 && NumUE > 0
    [UE(AttachList)] = Throughput(station.NameTag,UE(AttachList),NoiseDen,NoiseFigure,BWperRB,NumTimeSlots,NumFreqSlots);
    UEID = [UE(AttachList).ID];
    allocation = zeros(NumFreqSlots,NumTimeSlots);
    ABSix = find(station.ABS(1:NumTimeSlots) == 0); % The indices of subframes that are ABSs
    T = zeros(NumUE,NumTimeSlots+1); % Long-term average throughput of each user in each subframe.
    % The first column represents the past history, the second column represents the first subframe, ...
    ThroughputProfile = zeros(NumFreqSlots,NumTimeSlots,NumUE); % Record the throughput of each UE at each PRB
    for ue = 1:NumUE
        ThroughputProfile(:,:,ue) = [reshape(UE(AttachList(ue)).ThroughputPerPRB,[NumTimeSlots,NumFreqSlots])'];
    end
    
    % Allocate PRB according to (12.2)
    memory = 0; % This is to avoid unnecessary calculations of T
    for prb = 1:NumTimeSlots*NumFreqSlots
        % Find out which subframe it is
        sf = ceil(prb/NumFreqSlots);
        if sf ~= memory
            % Update the long-term average throughput for each UE
            for ue = 1:NumUE
                T(ue,sf+1) = (1-1/window)*T(ue,sf) + 1/window * sum([allocation(:,sf)==UEID(ue)].*ThroughputProfile(:,sf,ue));
            end
            memory = sf;
        end
        if isempty(find(ABSix==sf))
            % Get UE throughputs on that PRB
            PRB_ix = [prb prb+cumsum(NumFreqSlots*NumTimeSlots*ones(1,NumUE-1))];
            throughput = ThroughputProfile(PRB_ix)';
            % Find out to which UE should this PRB be allocated
            if ~isempty(find(T(:,sf+1)==0))
                nonzero_ix = find(T(:,sf+1));
                throughput(nonzero_ix) = 0;
                [nouse,ueix] = max(throughput);
            else
                [nouse,ueix] = max(throughput./T(:,sf+1));
            end
            allocation(prb) = UEID(ueix);
            % Update the long-term average throughput
            T(ueix,sf+1) = (1-1/window)*T(ueix,sf) + 1/window * sum([allocation(:,sf)==UEID(ueix)].*ThroughputProfile(:,sf,ueix));
        end
    end
    
    % Update the allocation patterns in station and UE
    station.UEinPRB = reshape(allocation',1,[]);
    for ue = 1:length(AttachList)
        UE(AttachList(ue)).Allocation = zeros(1,NumPRB);
        UE(AttachList(ue)).Allocation(find(station.UEinPRB==UEID(ue))) = 1;
    end
end