% max flow algorithm based on Ford-Fulkerson labeling algorithm
%
function [flow_matrix, S_plus, S_minus, J_plus, J_minus] = max_flow_FF(c1,c2,B)
%  N: number of agents
%  Q:  number of goods

zz = 1e-10;   % a positive number very close to zero

[N Q] = size(B);

%c1 =  c1/sum(c1);  % normalize 
%c2 = c2/sum(c2);   % normalize

flow1 = zeros(size(c1));
flow2 = zeros(size(c2));

flow_matrix = zeros(size(B));

aug_path = [1];   % bootstrap with an arbitrary non-empty vector
while(~isempty(aug_path)) 
% find an augmentation path
	[aug_path S T] = augmentation_path(find(c1>flow1+zz),B,flow_matrix>zz,find(c2>flow2+zz));
	
	if (~isempty(aug_path))   % if there is an augmentation path, increase the flow along this path
        if (mod(length(aug_path),2) == 1)
            error('error in length of aug. path'); % length(aug_path) must be even
        end
        % find the addition flow we can put in aug_path
        m= min( c1(aug_path(1)) - flow1(aug_path(1)), ...
                   c2(aug_path(end)) - flow2(aug_path(end)) );       
        for j = 2:2:(length(aug_path)-1)   % take the min. of all reverse link in aug_path
            m = min(m,flow_matrix(aug_path(j+1),aug_path(j)));
        end
        
        % increment the flow on the augmentation path
        flow1(aug_path(1)) = flow1(aug_path(1)) + m;
        flow2(aug_path(end)) = flow2(aug_path(end)) + m;
        for j = 1:2:length(aug_path)
            flow_matrix(aug_path(j),aug_path(j+1)) = ...
               flow_matrix(aug_path(j),aug_path(j+1)) + m; 
        end
        for j = 2:2:(length(aug_path)-1)
            flow_matrix(aug_path(j+1),aug_path(j)) = ...
               flow_matrix(aug_path(j+1),aug_path(j)) - m; 
        end
    end
end

max_flow = sum(flow1);
s_plus = S;
j_plus = T;

% find S_minus and J_minus by reversing the source and sink
[dummy j_minus s_minus] = augmentation_path(find(c2>flow2+zz),B',(flow_matrix>zz)',find(c1>flow1+zz));


% conversion to vector format
S_plus = zeros(1,N);    S_plus(s_plus) = ones(size(s_plus));
S_minus = zeros(1,N);   S_minus(s_minus) = ones(size(s_minus));
J_plus = zeros(1,Q);    J_plus(j_plus) = ones(size(j_plus));
J_minus = zeros(1,Q);   J_minus(j_minus) = ones(size(j_minus));



